

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% vx2dquad.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [t,nalp]=vx2dquad(c,G,delta);
% minimizes 2D problem f(x) = - 2 c^T x + x^T G x
% with safeguards in the nearly indefinite case.
% delta>=0 is a definiteness threshold
% 
% The optimum is at x=2^nalp*t, where t in [-1,1]^2
% and nalp=1 iff either G is nearly infinite,
% or the definite optimizer is in [-1,1]^2.
%
function [t,nalp]=vx2dquad(c,G,delta);

definite=0;
if G(1,1)>0,
  detG=G(1,1)*G(2,2)-G(1,2)^2;
  if detG>delta*G(1,2)^2,
    % G is sufficiently positive definite
    definite=1;
    % try unconstrained minimizer by Cramer's rule 
    t(1,1)=c(1)*G(2,2)-c(2)*G(2,1);
    t(2,1)=G(1,1)*c(2)-G(1,2)*c(1);
    t=t/detG; 
    tn=norm(t,inf);
    if ~isfinite(tn),% 111
      % no definite direction found
      disp('definite, but no definite direction found')
      definite=0;
    elseif tn<=2, % 222
      nalp=1;
    else, % 333
      % rescale direction
      nalp=ceil(log2(tn)); 
      t=t*2^(1-nalp);
    end;
    f2=t'*(G*t/2-c);
  end;
end;

if ~definite, % 444
  % G is not positive definite
  % find minimum constrained to the box [-1,1]^2
  f2=0; % objective value at t=0;
  for k=1:2,
    i=3-k;
    for tk=[-1,1],
      ci=c(i)-tk*G(i,k);
      Gii=G(i,i);
      % minimize -ci ti + Gii ti^2/2 s.t. ti in [-1,1]
      if Gii>abs(ci), 
        ti=ci/Gii;
        fti=-ci*ti/2;
      elseif ci>=0,  
        ti=1;
        fti=-ci+Gii/2;
      else          
        ti=-1;
        fti=ci+Gii/2;
      end;
      if fti<f2,
        f2=fti;
        t(i,1)=ti;t(k,1)=tk;
      end;
    end;
  end;
  if f2>=0, 
    % no good direction found
    G,c
    disp('bad fti');
    t=[1;1];nalp=1;
    return;
  end;
  nalp=10;
end;


