


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% vxinitf.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function f=vxinitf(x);
% evaluate function at x and add to history plot
%
% x(;,k)             % k-th argument
% f(1,k)             % k-th function value
%                    % but f(1,k)=inf if budget used up
%
% function vxinitf(fcn,nfmax,prt,drawstyle);
% initialize function interface
%
% fcn           % function handle of function to be called
% nfmax         % linit on number of function values
% prt           % print level (0=none)
% drawstyle     % two character drawing style (e.g., 'ro') 
%               % or empty (default, no drawing)
%
function f=vxinitf(x,nfmax0,prt0,drawstyle0);

persistent fcn nfmax prt drawstyle draw ndraw nfused fbest  

if nargout==0, 
  % initialize function interface
  fcn=x; 
  nfmax=nfmax0;
  prt=prt0;
  if nargin<4, 
    draw=0; 
  else   
    drawstyle=drawstyle0;     
    draw=~isempty(drawstyle);
  end;
  nfused=0;
  fbest=inf;
  ndraw=[0 fix(nfmax/100); 0 ceil(nfmax/100)];
  % ndraw = [<nf since last draw>     <wait until draw>;...
  %         [<nf since last drawnow>  <wait until drawnow>]
  return;
end; 

if nargin>1,
  error('too many input arguments to return a result');
end;

% loop over arguments
narg=size(x,2);
if 0%nfused>=nfmax,
  % function value budget used up
  f=1e150+zeros(1,narg);
  if 1, disp('nfmax exceeded'); end;
  prt=0;
  draw=0;
else 
  for k=1:narg,
    f(1,k)=fcn(x(:,k));
  end;
  nfused=nfused+narg;
end;

fmin=min(f);
if nfused>1, improved=(fmin<fbest);
else         improved=1;
end;
if improved,
  nfbest=nfused;fbest=fmin;
  if prt, 
    disp(['function value improved at nf=',num2str(nfused),...
          ' to f=',num2str(fmin)])
  end;
end;
