

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% vxline.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% line search decreasing fcn(xbest+alp*p) 
% 
% The initial alp is chosen automatically and scaling invariant,
% unless keepalp=1 where alp is used.
% p is modified if zero; then keepalp is set ot 0.
%
% updates xbest and fbest, but also returns the best tried value 
% abest of alp (including 0) relative to the original xbest, and the 
% best _newly_ evaluated argument x and its function value f.
%
% m-file called in main iteration of vx* routines
%
nalist=6;      % maximal number of trial points (at least 4 needed)
qlarge=10;     % asymmetry quotient considered large
alpeps=1e-12;  % limit accuracy for convex regularizartion
alpexp=1;      % damping factor for extrapolation
% alptol could also be varied
lineview=0;

abest=0;
a0=0;
convex=0;

% select initial step
if all(p==0)|~all(isfinite(p)),
  if prt, 
    disp('replace zero direction'); 
  end;
  p=iscale*(rand(n,1)-0.5);
  keepalp=0;
end;
ind=(xbest~=0);
ascale=min(abs(xbest(ind))./(abs(p(ind))+realmin));
alpdist=1e-8*ascale;
if keepalp,
  % use current alp
elseif any(ind), 
  alptol=10^(rand-2);
  alp=alptol*ascale;
else
  alp=iscale/norm(p,inf);
end;

alist=[0];
falist=[fbest];
for iline=1:nalist+1,
  nalp=length(alist);
  alps=norm(alist-abest,inf);
  if nalp>2,
    % cut new step to size
    alp=max(-100*alps,min(alp,100*alps));
  end;
  %if ~isfinite(alp) %%%%%%%%%%%%%%Achtung, möglicherweise Änderung%
    %xbest
    %p
    %ascale
    %alp=rand;
    %input('Next>');
  %end

  % function evaluation and list management
  x=xbest+alp*p;
  vxeval; % evaluate f=f(x) and keep books
  if done, break; end;
  alist=[alist,abest+alp];
  falist=[falist,f];
  newbest=(f<=fbest);
  if newbest,
    % update best point information
    fbest=f;
    xbest=xbest+alp*p;abest=abest+alp;
    nfbest=nfused;
  end;

  % quit line search?
  if convex & newbest | length(alist)>nalist, 
    % this may still be on a monotone stretch!
  
    % update dflist
    df=falist(1)-fbest;
    if df>0, dflist=[df,dflist(1:end-1)]; end;
%if nalp>3, fprintf('%i',nalp); end;
    % quit line search
    break; 
  end;

  

  % determine next step size

  nalp=length(alist);
  if nalp==2,
    % extrapolate to better side
    if falist(1)<falist(2), alp=alpexp*(alist(1)-alist(2));
    else                    alp=alpexp*(alist(2)-alist(1));
    end;
    continue;
  end;

  % sort points and find bracket
   
  [asorted,perm]=sort(alist-abest);
  fsorted=falist(perm);
  kk=find(asorted==0);
  a0=asorted(kk);
  f0=fsorted(kk); % best value
  if kk>1, 
    left=1;
    a1=asorted(kk-1);
    f1=fsorted(kk-1);
  else
    left=0;
    a1=asorted(kk+2);
    f1=fsorted(kk+2);
  end;
  if kk<nalp,
    right=1; 
    a2=asorted(kk+1);
    f2=fsorted(kk+1);
  else
    right=0;
    a2=asorted(kk-2);
    f2=fsorted(kk-2);
  end;
  bracket=(left & right);
  
  % linear kink interpolation applicable?
  if nalp==5 & kk==3, 
  % if kk>=3 & kk<=nalp-2,
    kink=0; % without central point
    vxkink;
    if kink,
      if lineview, disp('kink step'); end;
      continue;
    end;
  end;

  % geometric mean step applicable?
  if nalp>4,
    q=-a1/a2; 
    if lineview & q>0, qq=max(q,1/q), end;
    if q>qlarge, 
      alp=-a2*(1+rand*sqrt(q));
      if lineview, disp('geometric mean step A'); end;
      continue;
    elseif 1/q>qlarge, 
      alp=a1/(1+rand*sqrt(q));
      if lineview, disp('geometric mean step B'); end;
      continue;
    end;
  end;

  % try safeguarded quadratic interpolant 
  if nalp>3, 
    % restrict step by setting a gain target 
    if lineview, disp('restricted step'); end;
    linetarget=30*max(dflist(1),median(dflist));
  else       
    linetarget=inf;
  end;
  vxquad;
end; % for iline

% produce output refering to the original xbest
if abest==0,
  alp=a1;f=f1;x=xbest+a1*p;
else
  alp=abest;f=fbest;x=xbest;
end;
