

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% vxqr1.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% VXQR1 performs an approximate unconstrained minimization of a not 
% necessarily smooth function of many continuous arguments. 
% No gradients are needed.
% A limited amount of noise is tolerated. 
%
% The primary goal is to get in high dimensions quick improvements 
% with few function evaluations, not necessarily to find the true 
% global minimum.
%  
% A bounded initial search region must be specified but the actual 
% search is not confined to this region; thus the best point found 
% may be outside the search region.
%
% An example for how to use the program is in vxqr_driver.m
%
% The program was written by Arnold Neumaier (University of Vienna). 
% The current Version 1.0 is dated from June 18, 2010.
% 
% Please inform the author at Arnold.Neumaier@univie.ac.at if you make 
% serious use of this code. 
%  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [xbest,fbest,nfused,flist]=vxqr1(fcn,x,options);
%
% fcn            % objective function handle
%                % f=fcn(x) provides the function value f at x
% x(1:n,1)       % initial point
% options        % structure for options (NaN gives the default)
%                % goal-oriented options:
%   .iscale         % initial scale: initially, the solution is sought 
%                   % in a box of radius iscale around xinit
%   .stop           % stopping condition: stop if, for some k, 
%      .nftarget    % nf>=.nftarget(k) and
%      .ftarget     % fbest<=.ftarget(k),
%                   % but latest if nf>=max(nftarget)
%                   % to stop after nfmax function values, put
%                   % nftarget(1)=nfmax, ftarget(1)=inf;
%                   % to stop when fbest<=ftarget, put
%                   % nftarget(2)=0, ftarget(2)=ff;
%      .maxsec      % also stop after maxsec seconds
%   .prt            % print level for progress info? 
%                   % (0=none, 1=little, 2=more)   
%   .draw           % draw progress? (1=yes; assumes min f = 0)
%   .mark           % marking style (default: 'bo')   
%                % options for tuning performance:
%   .solver         % string defining the solver version
%                   % (default is 'default')
%   .gainfac        % factor for trial stepsize from expected gain 
%   .discfac        % discount factor for expected gain
%   .alptol         % tolerance in line search (defining lambdamin)
%   .alpfac         % tolerance in line search (defining first alp)
%   .regfac         % regularization factor for QR
%
% xbest                % best point found     
% fbest                % function value at xbest  
% nfused               % number of function evaluations used
% flist(:,k)=[nf;f]    % best function value f after nf function values
%                      % for nf=1,2,5,10,20,50,100,200,500,1000,...  
% 
function [xbest,fbest,nfused,flist]=vxmin(fcn,x,options);

n=size(x,1);
if nargin<3, options=[]; end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% set options and initialize
vxinit;

%%%%%%%%%%%%%%%
% main strategy
vxmain;

%%%%%%%%%%%%%%%%%%%%
% wrap up and finish
vxfinish;


